-- YGREC8/VHDL/proasic3v3/SRFF_PA3.vhdl
-- created mar. août 25 02:51:59 CEST 2020 by whygee@f-cpu.org
--
-- Here are various Set/Reset Flip-Flops, defined by the precedence
-- of the control signals and their active levels.
--
-- Precedence  Set  Reset  Macro
--            level level  name  Mapped to
--      Set     0     0    S0R0   AO1A
--      Set     0     1    S0R1   AO1C
--      Set     1     0    S1R0   AO1, AON21
--      Set     1     1    S1R1   AO1B, DLI1P1C1, AON21B
--    Reset     0     0    R0S0   OA1A
--    Reset     0     1    R1S0   OA1C
--    Reset     1     0    R0S1   OA1, OAN21
--    Reset     1     1    R1S1   OA1B, DLN1P1C1, OAN21B
--
--
-- The "backwards" component adds the necessary properties to the
-- loopback wire to allow a proper analysis of logic depth,
-- observability and controllability by breaking the feedback loop
-- of certain latches and flip-flops.

Library ieee;
    use ieee.std_logic_1164.all;
Library work;
    use work.all;
    use work.PA3_genlib.all;

entity backwards is
  port( A : in std_logic;
        Y : out std_logic);
end backwards;

architecture simple of backwards is
begin
  Y <= A; -- just a wire.
end simple;

-------------------- SET precedence --------------------

Library ieee;
    use ieee.std_logic_1164.all;
Library work;
    use work.all;
    use work.PA3_genlib.all;

entity S0R0 is
  port( S, R : in std_logic;
           Y : out std_logic);
end S0R0;

architecture simple of S0R0 is
  component backwards port( A : in std_logic; Y : out std_logic); end component;
  component AO1A port(A, B, C : in std_logic; Y : out std_logic); end component;
  signal i, o: std_logic := '0';
begin
  la: AO1A port map(A=>S, B=>i, C=>R, Y=>o);
  fb: backwards port map(A=>o, Y=>i);
  Y <= o;
end simple;


Library ieee;
    use ieee.std_logic_1164.all;
Library work;
    use work.all;
    use work.PA3_genlib.all;

entity S0R1 is
  port( S, R : in std_logic;
           Y : out std_logic);
end S0R1;

architecture simple of S0R1 is
  component backwards port( A : in std_logic; Y : out std_logic); end component;
  component AO1C port(A, B, C : in std_logic; Y : out std_logic); end component;
  signal i, o: std_logic := '0';
begin
  la: AO1C port map(A=>S, B=>i, C=>R, Y=>o);
  fb: backwards port map(A=>o, Y=>i);
  Y <= o;
end simple;


Library ieee;
    use ieee.std_logic_1164.all;
Library work;
    use work.all;
    use work.PA3_genlib.all;

entity S1R0 is
  port( S, R : in std_logic;
           Y : out std_logic);
end S1R0;

architecture simple of S1R0 is
  component backwards port( A : in std_logic; Y : out std_logic); end component;
  component AO1  port(A, B, C : in std_logic; Y : out std_logic); end component;
  signal i, o: std_logic := '0';
begin
  la: AO1 port map(A=>S, B=>i, C=>R, Y=>o);
  fb: backwards port map(A=>o, Y=>i);
  Y <= o;
end simple;


Library ieee;
    use ieee.std_logic_1164.all;
Library work;
    use work.all;
    use work.PA3_genlib.all;

entity S1R1 is
  port( S, R : in std_logic;
           Y : out std_logic);
end S1R1;

architecture simple of S1R1 is
  component backwards port( A : in std_logic; Y : out std_logic); end component;
  component AO1B port(A, B, C : in std_logic; Y : out std_logic); end component;
  signal i, o: std_logic := '0';
begin
  la: AO1B port map(A=>S, B=>i, C=>R, Y=>o);
  fb: backwards port map(A=>o, Y=>i);
  Y <= o;
end simple;


-------------------- RESET precedence --------------------

Library ieee;
    use ieee.std_logic_1164.all;
Library work;
    use work.all;
    use work.PA3_genlib.all;

entity R0S0 is
  port( S, R : in std_logic;
           Y : out std_logic);
end R0S0;

architecture simple of R0S0 is
  component backwards port( A : in std_logic; Y : out std_logic); end component;
  component OA1A port(A, B, C : in std_logic; Y : out std_logic); end component;
  signal i, o: std_logic := '0';
begin
  la: OA1A port map(A=>S, B=>i, C=>R, Y=>o);
  fb: backwards port map(A=>o, Y=>i);
  Y <= o;
end simple;


Library ieee;
    use ieee.std_logic_1164.all;
Library work;
    use work.all;
    use work.PA3_genlib.all;

entity R1S0 is
  port( S, R : in std_logic;
           Y : out std_logic);
end R1S0;

architecture simple of R1S0 is
  component backwards port( A : in std_logic; Y : out std_logic); end component;
  component OA1C port(A, B, C : in std_logic; Y : out std_logic); end component;
  signal i, o: std_logic := '0';
begin
  la: OA1C  port map(A=>S, B=>i, C=>R, Y=>o);
  fb: backwards port map(A=>o, Y=>i);
  Y <= o;
end simple;


Library ieee;
    use ieee.std_logic_1164.all;
Library work;
    use work.all;
    use work.PA3_genlib.all;

entity R0S1 is
  port( S, R : in std_logic;
           Y : out std_logic);
end R0S1;

architecture simple of R0S1 is
  component backwards port( A : in std_logic; Y : out std_logic); end component;
  component OA1  port(A, B, C : in std_logic; Y : out std_logic); end component;
  signal i, o: std_logic := '0';
begin
  la: OA1 port map(A=>S, B=>i, C=>R, Y=>o);
  fb: backwards port map(A=>o, Y=>i);
  Y <= o;
end simple;


Library ieee;
    use ieee.std_logic_1164.all;
Library work;
    use work.all;
    use work.PA3_genlib.all;

entity R1S1 is
  port( S, R : in std_logic;
           Y : out std_logic);
end R1S1;

architecture simple of R1S1 is
  component backwards port( A : in std_logic; Y : out std_logic); end component;
  component OA1B port(A, B, C : in std_logic; Y : out std_logic); end component;
  signal i, o: std_logic := '0';
begin
  la: OA1B port map(A=>S, B=>i, C=>R, Y=>o);
  fb: backwards port map(A=>o, Y=>i);
  Y <= o;
end simple;
